
import io
import sys
import traceback
from typing import List, Tuple

import pandas as pd

from db_copilot.analysis.analysis_com import Analyzer


class AutoAnalyzer(Analyzer):
    '''Auto analysis'''

    def get_analysis_info(self, df: pd.DataFrame, evidences: List[Tuple[str, str]], **kwargs) -> str:
        max_attempts = 3
        init_prompt = self.prompt_text_inner(df)
        current_prompt = init_prompt
        output = ''
        attempt = 1

        original_stdout = sys.stdout
        while attempt <= max_attempts:
            auto_analysis_response = ''
            try:
                auto_analysis_response = next(self.llm.complete(
                prompt=current_prompt, stop=None, temperature=0, stream=False))

                code_content = auto_analysis_response.split(
                    '```python')[1].split('```')[0].strip()
                buffer = io.StringIO()
                sys.stdout = buffer
                exec(code_content, {
                    'pd': pd,
                    'df': df
                })
                output = buffer.getvalue()
                buffer.close()
                break
            except Exception as e:
                current_prompt = f"""{init_prompt}
Your response is:
{auto_analysis_response}
There an error in your python code, it throws the following exception:
# Error: {type(e).__name__}
# Resaon: {str(e)}
# Stacktrace: {traceback.format_exc()}

Please try again.""".strip()
            finally:
                sys.stdout = original_stdout
            attempt += 1
        if not output:
            return ''
        description = f"""Here is the python code to perform the analysis:
{auto_analysis_response}""".strip()
        result = f"""After excuting the code, you can see the following results:
{output}""".strip()
        evidences.append((description, result))
        return f'{description}\n{result}'

    def prompt_text_inner(self, df: pd.DataFrame) -> str:
        return f"""{self.get_base_prompt(df)}
The current user query is:
{self.query}
what kind of analysis should I do?, please write a python code to do the analysis, assuming the input dataframe is df.
----------------
Here are some instructions on how to write the code:
- Please put all the python code in one python code block ```python...```
- The python code block should not contain any import statements.
  You can assume that, before the python code generated by you, the following packages have been imported:
  - import pandas as pd;
  Only the pandas package is allowed to be used in the generated python code block.
- Make sure the generated python code block can be executed successfully. 
- Print the results to the standard output.
Now please write the python code to do the analysis for {self.query}
""".strip()
